//
//  ViewController.swift
//  VoltageOutput
//
//  Created by Phidgets on 2018-01-30.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = VoltageOutput()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var voltageLabel: UILabel!
    @IBOutlet var voltageSlider: UISlider!
    @IBOutlet var enableTitle: UILabel!
    @IBOutlet var enableSegment: UISegmentedControl!
    @IBOutlet var voltageRangeTitle: UILabel!
    @IBOutlet var voltageRangeSegment: UISegmentedControl!
    @IBOutlet var stackView: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! VoltageOutput
            
            let deviceID = try attachedDevice.getDeviceID()
            let minVoltage = try Float(attachedDevice.getMinVoltage())
            let maxVoltage = try Float(attachedDevice.getMaxVoltage())
            let voltage = try Float(attachedDevice.getVoltage())
            var enabled:Bool = false
            var range:VoltageOutputRange? = nil
            
            switch(deviceID){
            case DeviceID.PN_1002:
                fallthrough
            case DeviceID.PN_OUT1000:
                enabled = try attachedDevice.getEnabled()
                DispatchQueue.main.async {
                    self.enableSegment.selectedSegmentIndex = enabled ? 0 : 1
                    self.updateGUI(isEnabledHidden: false, isRangeHidden: true)
                }
                break
            case DeviceID.PN_OUT1001:
                fallthrough
            case DeviceID.PN_OUT1002:
                range = try attachedDevice.getVoltageOutputRange()
                DispatchQueue.main.async {
                    self.voltageRangeSegment.selectedSegmentIndex = (range == VoltageOutputRange.volts_5) ? 0 : 1
                    self.updateGUI(isEnabledHidden: true, isRangeHidden: false)
                }
                break
            default:
                break
            }

            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.voltageSlider.minimumValue = minVoltage
                self.voltageSlider.maximumValue = maxVoltage
                self.voltageSlider.value = voltage
                self.voltageLabel.text = String(format: "%.2f V", voltage)
            }

        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }

    //MARK: GUI controls
    @IBAction func setVoltage(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setVoltage(Double(slider.value))
            DispatchQueue.main.async {
                self.voltageLabel.text = String(format:"%.2f V", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setEnabled(_ sender: Any) {
        do{
            let segment = sender as! UISegmentedControl
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setEnabled(true)
                break
            case 1:
                try ch.setEnabled(false)
                break
            default:
                break
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setVoltageOutputRange(_ sender: Any) {
        do{
            let segment = sender as! UISegmentedControl
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setVoltageOutputRange(VoltageOutputRange.volts_5)
                break
            case 1:
                try ch.setVoltageOutputRange(VoltageOutputRange.volts_10)
                break
            default:
                break
            }
            
            let minimumVoltage = try Float(ch.getMinVoltage())
            let maximumVoltage = try Float(ch.getMaxVoltage())
            let voltage = try Float(ch.getVoltage())
            
            DispatchQueue.main.async {
                self.voltageSlider.minimumValue = minimumVoltage
                self.voltageSlider.maximumValue = maximumVoltage
                self.voltageSlider.value = voltage
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func updateGUI(isEnabledHidden:Bool, isRangeHidden:Bool){
        DispatchQueue.main.async {
            self.enableTitle.isHidden = isEnabledHidden
            self.enableSegment.isHidden = isEnabledHidden
            self.voltageRangeTitle.isHidden = isRangeHidden
            self.voltageRangeSegment.isHidden = isRangeHidden
        }
    }

}

